# Namespace wildcard. Example usage: f'{NS_WC}MD_Metadata']'
NS_WC = "*[local-name()='"

GMD_NAMESPACE = "http://www.isotc211.org/2005/gmd"
GCO_NAMESPACE = "http://www.isotc211.org/2005/gco"
GMX_NAMESPACE = "http://www.isotc211.org/2005/gmx"
SRV_NAMESPACE = "http://www.isotc211.org/2005/srv"
GML_3_1_1_NAMESPACE = "http://www.opengis.net/gml"
GML_3_2_2_NAMESPACE = "http://www.opengis.net/gml/3.2"
XLINK_NAMESPACE = "http://www.w3.org/1999/xlink"
CSW_2_0_2_NAMESPACE = "http://www.opengis.net/cat/csw/2.0.2"
WFS_1_1_2_NAMESPACE = "http://www.opengis.net/wfs"
WFS_2_0_0_NAMESPACE = "http://www.opengis.net/wfs/2.0"
WMS_1_3_0_NAMESPACE = "http://www.opengis.net/wms"
OWS_NAMESPACE = "http://www.opengis.net/ows"
OWS_1_1_NAMESPACE = "http://www.opengis.net/ows/1.1"
FES_2_0_NAMEPSACE = "http://www.opengis.net/fes/2.0"
OGC_NAMESPACE = "http://www.opengis.net/ogc"

XSI_NAMESPACE = "http://www.w3.org/2001/XMLSchema-instance"
